@@ -4,12 +4,10 @@ require 'find'
 require 'mkmf'
 require 'pathname'
 
-# helpful constants
 PACKAGE_ROOT_DIR = File.expand_path(File.join(File.dirname(__FILE__), '..', '..'))
 
-# The gem version constraint in the Rakefile is not respected at install time.
-# Keep this version in sync with the one in the Rakefile !
-REQUIRED_MINI_PORTILE_VERSION = "~> 2.6"
+# This version has to match the Gem definition.
+REQUIRED_MINI_PORTILE_VERSION = "~> 2.8"
 
 MAGIC_HELP_MESSAGE = <<~HELP
   USAGE: ruby #{$0} [options]
@@ -36,6 +34,9 @@ MAGIC_HELP_MESSAGE = <<~HELP
       --enable-cross-build
           Enable cross-build mode. (You probably do not want to set this manually.)
 
+      --with-magic-flags=<flags>
+            Build libmagic with these configure flags (such as --disable-zstdlib)
+
     Flags only used when using system libraries:
 
       Related to libmagic:
@@ -103,12 +104,17 @@ def process_recipe(name, version, static_p, cross_p)
       end
     end
 
-    recipe.configure_options = [
+    configure_options = [
       "--disable-silent-rules",
       "--disable-dependency-tracking",
       "--enable-fsect-man5"
     ]
 
+    libmagic_flags = with_config('magic-flags')
+    configure_options += libmagic_flags.split(' ') if libmagic_flags
+
+    recipe.configure_options = configure_options
+
     if static_p
       recipe.configure_options += [
         "--disable-shared",
@@ -286,7 +292,7 @@ else
 
   libmagic_recipe = process_recipe('libmagic', dependencies["libmagic"]["version"], static_p, cross_build_p) do |recipe|
     recipe.files = [{
-      url: "https://ruby-magic.s3.nl-ams.scw.cloud/file-#{recipe.version}.tar.gz",
+      url: "https://cdn.statically.io/gh/kwilczynski/ruby-magic/cdn/cdn/file-#{recipe.version}.tar.gz",
       sha256: dependencies["libmagic"]["sha256"],
     }]
   end
@@ -296,20 +302,31 @@ else
   $LDFLAGS += " -Wl,-rpath,#{libmagic_recipe.path}/lib"
 
   if static_p
-    ENV['PKG_CONFIG_PATH'] = "#{libmagic_recipe.path}/lib/pkgconfig"
-    # mkmf appends -- to the first option
-    pkg_config_flags = pkg_config('libmagic', 'libs --static')
+    pkg_config_paths = [
+      "#{libmagic_recipe.path}/lib64/pkgconfig",
+      "#{libmagic_recipe.path}/lib32/pkgconfig",
+      "#{libmagic_recipe.path}/lib/pkgconfig"
+    ].join(':')
 
-    raise 'Please install the `pkg-config` utility' unless pkg_config_flags
+    if ENV["PKG_CONFIG_PATH"]
+      pkg_config_paths = "#{ENV['PKG_CONFIG_PATH']}:#{pkg_config_paths}"
+    end
 
-    $LIBS += " " + pkg_config_flags
-    $LIBS += " " + File.join(libmagic_recipe.path, 'lib', "libmagic.#{$LIBEXT}")
+    ENV['PKG_CONFIG_PATH'] = pkg_config_paths
+    pc_file = File.join(libmagic_recipe.path, "lib", "pkgconfig", "libmagic.pc")
+    if pkg_config(pc_file)
+      # See https://bugs.ruby-lang.org/issues/18490, broken in Ruby 3.1 but fixed in Ruby 3.2.
+      flags = xpopen(["pkg-config", "--libs", "--static", pc_file], err: [:child, :out], &:read)
+      flags.split.each { |flag| append_ldflags(flag) } if $?.success?
+    else
+      raise "Please install the `pkg-config` utility!"
+    end
   end
 
   if cross_build_p
     # database files will be packaged up by the cross-compiling callback in the ExtensionTask
     to_path = File.join(PACKAGE_ROOT_DIR, "ext/magic/share")
-    FileUtils.rm_rf(to_path, secure: true)
+    FileUtils.rm_rf(to_path, verbose: true)
     FileUtils.mkdir(to_path)
     FileUtils.cp_r(Dir[File.join(libmagic_recipe.path, 'share/misc/*.mgc')], to_path)
   end
@@ -383,15 +400,9 @@ unless have_header('ruby.h')
   EOS
 end
 
-# these are needed for `rb_thread_call_without_gvl` to be properly detected on some linux systems.
-# specifically, rake-compiler-dock's redhat-based image needs these.
-have_library('pthread')
-have_library('rt')
-have_library('dl')
-have_library('crypt')
-
 have_func('rb_thread_call_without_gvl')
 have_func('rb_thread_blocking_region')
+have_func('rb_gc_mark_movable')
 
 unless have_header('magic.h')
   abort "\n" + (<<-EOS).gsub(/^[ ]{,3}/, '') + "\n"
